
//######################################################################################
//#                                                                                    #
//# Author       : Sadek A. Nehmeh                                                     #
//# Institution  : Memorial Sloan-Kettering Cancer Center                              #
//# Email        : nehmehs@mskcc.org                                                   # 
//#                                                                                    #
//# Program      : GEAdvance3D_Sinogram_Bin.c                                          # 
//# Version      : V.1.0 (12/13/2004)                                                  #
//#                                                                                    #   
//# Objective    : To read the coincidences TTree from the .root file, and generate the#
//#                correpsonding Michelogram and Projection planes file according to GE#
//#                format.                                                             #
//#                                                                                    #
//# Scanners     : GE Advance, GE Discovery LS, GE Discovery ST                        #
//#                                                                                    # 
//# Input        : ROOT file                                                           #
//#                                                                                    #
//#                                                                                    #
//#                                                                                    #
//# Output       : 1- Michelogram.ima                                                  #
//#                2- MichelogramF.ima  (for STIR based reconstructions)               #
//#                3- Projections.ima   (for GE Advance/DLS based reconstructions)     #
//#                4- ProjectionsF.ima                                                 #
//#                Format: "F" => float (4 Bytes), short int (2 Bytes) otherwise.      #
//#                                                                                    #
//#                                                                                    #
//######################################################################################



{
  gROOT->Reset();


  //######################################################################################
  //#                              Define the scanner geometry parameters                #
  //######################################################################################

#define  NUMBER_OF_RINGS               18 //     (EQUALS TO  18  FOR Advance/DLS)
#define  CRYSTALS_PER_RING            672 //     (equals to 672  for Advance/DLS; 420 for Discovery ST)
#define  SINOGRAM_WIDTH               283 //     (equals to 283  for Advance/DLS; 249 for Discovery ST)
#define  MAX_THETA                      5 //     (equals to   5  for Advance/DLS,  11 for Discovery ST)
#define  NUMBER_OF_PROJECTION_PLANES  265 //     (equals to 265  for Advance/DLS; 553 for Discovery ST)
#define  FOV                          550 //     (equals to 55cm for Advance/DLS)
#define  SLICES_PER_FOV                35 //     (equals to 35   for Advance/DLS)



  
  //######################################################################################
  //#                               Loop over the .root file in the directory "PATH"     #
  //######################################################################################
  char Root_File[250];
  int nRUNS;
  
  printf("Please enter the number of runs: ");
  scanf("%d", &nRUNS);
  
  TChain chain("Coincidences");
  for (Int_t runID =0; runID<=nRUNS; runID++)
    {
      sprintf(Root_File, "<PATH>/run%d.root",runID);      
      chain.Add(Root_File);
    }  
 
  //#####################################################################################
  //#                Declaration of leaves types - TTree Coincidences                   #
  //#####################################################################################

  
  Float_t         axialPos;
  Char_t          comptVolName1[40];
  Char_t          comptVolName2[40];
  Int_t           compton1;
  Int_t           compton2;
  Int_t           crystalID1;
  Int_t           crystalID2;
  Int_t           comptonPhantom1;
  Int_t           comptonPhantom2;
  Float_t         energy1;
  Float_t         energy2;   
  Int_t           eventID1;
  Int_t           eventID2;
  Float_t         globalPosX1;
  Float_t         globalPosX2;
  Float_t         globalPosY1;
  Float_t         globalPosY2;      
  Float_t         globalPosZ1;
  Float_t         globalPosZ2;
  Int_t           layerID1;
  Int_t           layerID2;
  Int_t           moduleID1;
  Int_t           moduleID2;
  Float_t         rotationAngle;
  Int_t           rsectorID1;
  Int_t           rsectorID2;
  Int_t           runID;
  Float_t         sinogramS;
  Float_t         sinogramTheta;
  Int_t           sourceID1;
  Int_t           sourceID2;
  Float_t         sourcePosX1;
  Float_t         sourcePosX2;
  Float_t         sourcePosY1;
  Float_t         sourcePosY2;
  Float_t         sourcePosZ1;
  Float_t         sourcePosZ2;
  Int_t           submoduleID1;
  Int_t           submoduleID2;
  Double_t         time1;
  Double_t         time2;
  
  


//######################################################################################
//#                          Declaration of some variables                             #
//######################################################################################

  Int_t                   phi, theta,  count;
  Int_t                   i,j,k,l,entry, ring1, ring2, ring;
  Int_t                   module1, module2;             

  Float_t                 ProjectionsF[CRYSTALS_PER_RING/2][NUMBER_OF_PROJECTION_PLANES][SINOGRAM_WIDTH];
  Float_t                 MichelogramF[NUMBER_OF_RINGS][NUMBER_OF_RINGS][CRYSTALS_PER_RING/2][SINOGRAM_WIDTH]; 

  unsigned short          ans;
  unsigned short          Projections[CRYSTALS_PER_RING/2][NUMBER_OF_PROJECTION_PLANES][SINOGRAM_WIDTH];
  unsigned short          Michelogram[NUMBER_OF_RINGS][NUMBER_OF_RINGS][CRYSTALS_PER_RING/2][SINOGRAM_WIDTH] ;
 
  FILE *ProjectionsFile;
  FILE *ProjectionsFFile;
  FILE *MichelogramFile;
  FILE *MichelogramFFile;  
  
  ProjectionsFile   = fopen("Projections.ima","wb");
  ProjectionsFFile  = fopen("ProjectionsF.ima","wb");
  MichelogramFile   = fopen("Michelogram.ima","wb");
  MichelogramFFile  = fopen("MichelogramF.ima","wb");
 
  
  //####################################################################################
  //#                        Set branch addresses - TTree Coincicences                 #
  //####################################################################################
  
  
  Coincidences->SetBranchAddress("axialPos",&axialPos);
  Coincidences->SetBranchAddress("comptVolName1",&comptVolName1);
  Coincidences->SetBranchAddress("comptVolName2",&comptVolName2);
  Coincidences->SetBranchAddress("comptonCrystal1",&compton1);
  Coincidences->SetBranchAddress("comptonCrystal2",&compton2);
  Coincidences->SetBranchAddress("crystalID1",&crystalID1);
  Coincidences->SetBranchAddress("crystalID2",&crystalID2);
  Coincidences->SetBranchAddress("comptonPhantom1",&comptonPhantom1);
  Coincidences->SetBranchAddress("comptonPhantom2",&comptonPhantom2);
  Coincidences->SetBranchAddress("energy1",&energy1);
  Coincidences->SetBranchAddress("energy2",&energy2);   
  Coincidences->SetBranchAddress("eventID1",&eventID1);
  Coincidences->SetBranchAddress("eventID2",&eventID2);
  Coincidences->SetBranchAddress("globalPosX1",&globalPosX1);
  Coincidences->SetBranchAddress("globalPosX2",&globalPosX2);
  Coincidences->SetBranchAddress("globalPosY1",&globalPosY1);
  Coincidences->SetBranchAddress("globalPosY2",&globalPosY2);      
  Coincidences->SetBranchAddress("globalPosZ1",&globalPosZ1);
  Coincidences->SetBranchAddress("globalPosZ2",&globalPosZ2);
  Coincidences->SetBranchAddress("layerID1",&layerID1);
  Coincidences->SetBranchAddress("layerID2",&layerID2);
  Coincidences->SetBranchAddress("moduleID1",&moduleID1);
  Coincidences->SetBranchAddress("moduleID2",&moduleID2);
  Coincidences->SetBranchAddress("rotationAngle",&rotationAngle);
  Coincidences->SetBranchAddress("rsectorID1",&rsectorID1);
  Coincidences->SetBranchAddress("rsectorID2",&rsectorID2);
  Coincidences->SetBranchAddress("runID",&runID);
  Coincidences->SetBranchAddress("sinogramS",&sinogramS);
  Coincidences->SetBranchAddress("sinogramTheta",&sinogramTheta);
  Coincidences->SetBranchAddress("sourceID1",&sourceID1);
  Coincidences->SetBranchAddress("sourceID2",&sourceID2);
  Coincidences->SetBranchAddress("sourcePosX1",&sourcePosX1);
  Coincidences->SetBranchAddress("sourcePosX2",&sourcePosX2);
  Coincidences->SetBranchAddress("sourcePosY1",&sourcePosY1);
  Coincidences->SetBranchAddress("sourcePosY2",&sourcePosY2);
  Coincidences->SetBranchAddress("sourcePosZ1",&sourcePosZ1);
  Coincidences->SetBranchAddress("sourcePosZ2",&sourcePosZ2);
  Coincidences->SetBranchAddress("submoduleID1",&submoduleID1);
  Coincidences->SetBranchAddress("submoduleID2",&submoduleID2);
  Coincidences->SetBranchAddress("time1",&time1);
  Coincidences->SetBranchAddress("time2",&time2);
  
  /////////////STAT////////   
  gStyle -> SetStatW(0.28);
  gStyle -> SetStatH(0.3);
  gStyle -> SetStatColor(41);   
  gStyle -> SetStatX(1);
  gStyle -> SetStatY(1);   
  gStyle -> SetStatFont(42);
  gStyle->SetOptStat(0);
  gStyle->SetOptFit(0);
  /////////////////////////
  
  //###################################################################################
  //#                                Book Some Histograms                             #
  //###################################################################################
  
  // Book the Projection planes histograms...
  //----------------------------------------
  TH2F *proj0 =  new TH2F("proj0","proj0",   SINOGRAM_WIDTH,-FOV,+FOV,  SLICES_PER_FOV,0.,SLICES_PER_FOV);
  TH2F *proj1 =  new TH2F("proj1","proj1",   SINOGRAM_WIDTH,-FOV,+FOV,  NUMBER_OF_PROJECTION_PLANES,0.,NUMBER_OF_PROJECTION_PLANES);
  TH2F *proj2 =  new TH2F("proj2","proj2",   SINOGRAM_WIDTH,-FOV,+FOV,  NUMBER_OF_PROJECTION_PLANES,0.,NUMBER_OF_PROJECTION_PLANES);
  TH2F *proj3 =  new TH2F("proj3","proj3",   SINOGRAM_WIDTH,-FOV,+FOV, NUMBER_OF_PROJECTION_PLANES,0.,NUMBER_OF_PROJECTION_PLANES);
  TH2F *proj4 =  new TH2F("proj4","proj4",   SINOGRAM_WIDTH,-FOV,+FOV, NUMBER_OF_PROJECTION_PLANES,0.,NUMBER_OF_PROJECTION_PLANES);
  TH2F *proj5 =  new TH2F("proj5","proj5",   SINOGRAM_WIDTH,-FOV,+FOV, NUMBER_OF_PROJECTION_PLANES,0.,NUMBER_OF_PROJECTION_PLANES);
  TH2F *proj6 =  new TH2F("proj6","proj6",   SINOGRAM_WIDTH,-FOV,+FOV, NUMBER_OF_PROJECTION_PLANES,0.,NUMBER_OF_PROJECTION_PLANES);
  TH2F *proj7 =  new TH2F("proj7","proj7",   SINOGRAM_WIDTH,-FOV,+FOV, NUMBER_OF_PROJECTION_PLANES,0.,NUMBER_OF_PROJECTION_PLANES);
  TH2F *proj8 =  new TH2F("proj8","proj8",   SINOGRAM_WIDTH,-FOV,+FOV, NUMBER_OF_PROJECTION_PLANES,0.,NUMBER_OF_PROJECTION_PLANES);
  TH2F *proj9 =  new TH2F("proj9","proj9",   SINOGRAM_WIDTH,-FOV,+FOV, NUMBER_OF_PROJECTION_PLANES,0.,NUMBER_OF_PROJECTION_PLANES);
  TH2F *proj10 = new TH2F("proj10","proj10", SINOGRAM_WIDTH,-FOV,+FOV, NUMBER_OF_PROJECTION_PLANES,0.,NUMBER_OF_PROJECTION_PLANES);
  TH2F *proj11 = new TH2F("proj11","proj11", SINOGRAM_WIDTH,-FOV,+FOV, NUMBER_OF_PROJECTION_PLANES,0.,NUMBER_OF_PROJECTION_PLANES);
  
  // Book the sinogram histogram (this is the sum of all histogram)
  //--------------------------------------------------------------
  TH2F *sinogram_sum = new TH2F("sinogram","Sum of Sinograms", SINOGRAM_WIDTH,0.,+SINOGRAM_WIDTH, CRYSTALS_PER_RING/2,0.,CRYSTALS_PER_RING/2.);
  
  
  // Save the statistics per sinogram...
  //------------------------------------
  TH1F *sino_counts = new TH1F("Sinograms Statistics","Statistics per Sinogram ",(NUMBER_OF_RINGS*NUMBER_OF_RINGS),-0.5,((NUMBER_OF_RINGS*NUMBER_OF_RINGS)-0.5));
  
  //Int_t nentries = Coincidences->GetEntries();
  Int_t Trues, Randoms;
  Int_t nentries = chain.GetEntries();
  Int_t nbytes = 0, nrandom = 0, nscatter = 0, ntrue = 0;
  Float_t PI = 3.141592654;
  
  
  
  //#######################################################################################################
  //#                       SINOGRAMS AND PROJECTION PLANES BINNING                                       #
  //#######################################################################################################
  
  
  Int_t sinogram_counts[NUMBER_OF_RINGS][NUMBER_OF_RINGS];
  
  for (i=0; i<NUMBER_OF_RINGS; i++)
    for (j=0;j<NUMBER_OF_RINGS; j++) 
      sinogram_counts[i][j] = 0;
  
  
  Trues =0;
  Randoms =0;
  
  // Print the total number of coincidences from all the runs...
  //------------------------------------------------------------
  printf("\nTotal Number of Coincidence Events:= %d \n",nentries ); 
  
  //Loop for each event in the TTree Coincidences
  //---------------------------------------------
  
  Int_t v, u, psi;
  
  for (Int_t i=0; i<nentries; i++) //nentries
    {      
      if ((i%100000) == 0)
	printf("... %d\n",i);       
      nbytes += chain.GetEntry(i);
      
      // Update the number of Trues and Randoms...
      //------------------------------------------
      if (eventID1 == eventID2)
	Trues += 1;
      else
	Randoms += 1;            
      
     
     
     if ((moduleID1%2) == 0)
       module1 = moduleID1/2;
     else
       module1 = (moduleID1-1)/2;
     
     if ((moduleID2%2) == 0)
       module2 = moduleID2/2;
     else
       module2 = (moduleID2-1)/2;
   

     //  Identify the ring#...
     //-----------------------
     ring1 = ((Int_t)(crystalID1/6)) + module1*6 ;
     ring2 = ((Int_t)(crystalID2/6)) + module2*6 ;                
         
     
   
     //  Identify the crystal# for each of the two gammas...
     //-----------------------------------------------------
     Int_t crystal1,crystal2;
     
     crystal1 = (rsectorID1 * 12) + ((moduleID1%2)*6) + (crystalID1%6);
     crystal2 = (rsectorID2 * 12) + ((moduleID2%2)*6) + (crystalID2%6);    
     

     // Bin the sinogram and projection planes parameters (u, v, phi, and theta) according to GEMS definitions...
     //----------------------------------------------------------------------------------------------------------

     //######################################################################################
     //#       Michelogram and Projection Pleanes binning according to GEMS format          #
     //######################################################################################
     
     int flip, swap;
     
     if (crystal1 < crystal2)
       flip=0;
     else
       flip=1;
     
     phi = ((( crystal1 +  crystal2 + CRYSTALS_PER_RING/2) % CRYSTALS_PER_RING)/2);
     
     if ( ((crystal1 +  crystal2) < ( (3*CRYSTALS_PER_RING/2) -1) ) && ( (crystal1 +  crystal2) >=  (CRYSTALS_PER_RING/2)  ))            
       u     = abs(crystal1 -  crystal2) + (SINOGRAM_WIDTH - 1)/2 - CRYSTALS_PER_RING/2;
     else
       {
	 u = ( (SINOGRAM_WIDTH - 1)/2) + (CRYSTALS_PER_RING/2) - abs(crystal1 -  crystal2);
	 flip = 1 - flip;
       }     
     
     if (flip) 
       {
	 swap = ring1;
	 ring1 = ring2;
	 ring2 = swap;
       }
     
     theta = (ring1 - ring2)/2;
     
     if ( abs(theta) <= MAX_THETA )   
       {
         if ( abs(ring1 - ring2) <= 1)                      
           {
             v = (ring1 + ring2);
           }
	 
         if ( (ring1 - ring2) > 1)
           {
             psi = (ring1 - ring2)/2;
             v   = ring1 + ring2 + (4*psi -2)*NUMBER_OF_RINGS - (4*(psi*psi) - 1);
           }
	 
         if ( (ring1 - ring2) < -1)
           {
             psi = (ring2 - ring1)/2;
             v   = ring1 + ring2 + 4*psi*NUMBER_OF_RINGS - (4*(psi*psi) + 4*psi);            
           } 
       

	 //################################ End of GEMS binning ###############################
	 
	 // Update the different arrays...
	 //-------------------------------
	 
	 
	 
	 if (u>=0 && u<283)
	   {
	     sinogram_counts[ring2][ring1] += 1;
	     Projections[phi][v][u]  += 1;		 
	     ProjectionsF[phi][v][u]  += 1.;	    
	     Michelogram[ring2][ring1][phi][u] += 1;
	     MichelogramF[ring2][ring1][phi][u] += 1.;
	     sinogram_sum->Fill(u,phi);	 
	     
	     // Fill the projection  planes histograms...
	     //------------------------------------------
	     if (theta ==0 )	   
	       proj0->Fill(u,v); 	 	   
	     if (theta==0)	 
	       proj1->Fill(u,v);    
	     if (theta==1)	 
	       proj2->Fill(u,v);   
	     if (theta==-1)	 
	       proj3->Fill(u,v); 
	     if (theta==2)	 
	       proj4->Fill(u,v);
	     if (theta==-2)	 
	       proj5->Fill(u,v);
	     if (theta==3)	 
	       proj6->Fill(u,v);
	     if (theta==-3)	 
	       proj7->Fill(u,v); 
	     if (theta==4)	 
	       proj8->Fill(u,v);
	     if (theta==-4)	 
	       proj9->Fill(u,v);
	     if (theta==5)	 
	       proj10->Fill(u,v);
	     if (theta==-5)	 
	       proj11->Fill(u,v);
	   }         
       }
    }

 
 
  // Write the data to disk, and then close all opened files...
  //-----------------------------------------------------------
  ans = fwrite(Projections,2,(CRYSTALS_PER_RING/2*NUMBER_OF_PROJECTION_PLANES*SINOGRAM_WIDTH),ProjectionsFile);
  ans = fwrite(ProjectionsF,4,(CRYSTALS_PER_RING/2*NUMBER_OF_PROJECTION_PLANES*SINOGRAM_WIDTH),ProjectionsFFile);
  ans = fwrite(Michelogram,2,(NUMBER_OF_PROJECTION_PLANES*CRYSTALS_PER_RING/2*SINOGRAM_WIDTH),MichelogramFile);
  ans = fwrite(MichelogramF,4,(324*CRYSTALS_PER_RING/2*SINOGRAM_WIDTH),MichelogramFFile);
  
  fclose(MichelogramFile);
  fclose(MichelogramFFile);
  fclose(ProjectionsFile);
  fclose(ProjectionsFFile);
  
  
  
  // fill in the counts per sinogram histogram...
  //---------------------------------------------
  count = 0;
  for (i=0; i<NUMBER_OF_RINGS; i++)
    {
      for (j=0; j<NUMBER_OF_RINGS; j++) 
	{ 
	  for (l=0; l<sinogram_counts[i][j]; l++)
	    sino_counts->Fill(count);
	  count += 1;      
	}
    }
  
  //show some results...
  //--------------------
  c1 = new TCanvas("c1","Sinogram Average",0,0,700,700);

  c1->SetFillColor(0);
  c1->SetBorderMode(0);
  
  proj1->Draw();
  proj2.SetMarkerColor(2);
  proj2->Draw("same");
  proj3->Draw("same");
  proj4.SetMarkerColor(2);
  proj4->Draw("same");
  proj5->Draw("same");
  proj6.SetMarkerColor(2);
  proj6->Draw("same");
  proj7->Draw("same");
  proj8.SetMarkerColor(2);
  proj8->Draw("same");
  proj9->Draw("same");
  proj10.SetMarkerColor(2);
  proj10->Draw("same");
  proj11->Draw("same");
   
  
}

	
//############################# END OF GEAdvance3D_Sinogram_Bin.C ###############################
